@@ -1,5 +1,5 @@
 # Interface between GDB and Insight.
-# Copyright 1997, 1998, 1999 Cygnus Solutions
+# Copyright 1997, 1998, 1999, 2001 Red Hat, Inc.
 #
 # This program is free software; you can redistribute it and/or modify it
 # under the terms of the GNU General Public License (GPL) as published by
@@ -16,14 +16,27 @@
 global gdbtk_state
 set gdbtk_state(busyCount) 0
 
+# *** DEPRECATED: Use GDBEventHandler::breakpoint instead.
 # This is run when a breakpoint changes.  The arguments are the
 # action, the breakpoint number, and the breakpoint info.
-define_hook gdb_breakpoint_change_hook
+#define_hook gdb_breakpoint_change_hook
 
+# *** DEPRECATED: Use GDBEventHandler::set_variable instead.
 # This is run when a `set' command successfully completes in gdb.  The
 # first argument is the gdb variable name (as a Tcl list).  The second
 # argument is the new value.
-define_hook gdb_set_hook
+#define_hook gdb_set_hook
+
+# ------------------------------------------------------------
+#  PROC:  gdbtk_tcl_set_variable - A "set" command was issued
+#          in gdb to change an internal variable. Notify
+#          gui.
+# ------------------------------------------------------------
+proc gdbtk_tcl_set_variable {var val} {
+  set e [SetVariableEvent \#auto -variable $var -value $val]
+  GDBEventHandler::dispatch $e
+  delete object $e
+}
 
 ####################################################################
 #                                                                  #
@@ -34,6 +47,7 @@ define_hook gdb_set_hook
 # For debugging purposes, please put debug statements at the very
 # beginning and ends of all GUI state hooks.
 
+# *** DEPRECATED: Use GDBEventHandler::busy instead.
 # GDB_BUSY_HOOK
 #   This hook is used to register a callback when the UI should
 #   be disabled because the debugger is either busy or talking
@@ -41,8 +55,9 @@ define_hook gdb_set_hook
 #
 #   All callbacks should disable ALL user input which could cause
 #   any state changes in either the target or the debugger.
-define_hook gdb_busy_hook
+#define_hook gdb_busy_hook
 
+# *** DEPRECATED: Use GDBEventHandler::idle instead.
 # GDB_IDLE_HOOK
 #   This hook is used to register a callback when the UI should
 #   be enabled because the debugger is no longer busy.
@@ -52,10 +67,11 @@ define_hook gdb_busy_hook
 #   time delays when enabling the UI.
 define_hook gdb_idle_hook
 
+# *** DEPRECATED: Use GDBEventHandler::update instead.
 # GDB_UPDATE_HOOK
 #   This hook is used to register a callback to update the widget
 #   when debugger state has changed.
-define_hook gdb_update_hook
+#define_hook gdb_update_hook
 
 # GDB_NO_INFERIOR_HOOK
 #   This hook is used to register a callback which should be invoked
@@ -109,19 +125,17 @@ proc gdbtk_tcl_preloop { } {
 
 
 # ------------------------------------------------------------------
-#  PROCEDURE:  gdbtk_busy - run all busy hooks
+#  PROCEDURE:  gdbtk_busy - Dispatch a busy event
 #
 #         Use this procedure from within GUI code to indicate that
 #         the debugger is busy, either running the inferior or
-#         talking to the target. This will call all the registered
-#         gdb_busy_hook's.
+#         talking to the target.
 # ------------------------------------------------------------------
 proc gdbtk_busy {} {
 
-  set err [catch {run_hooks gdb_busy_hook} txt]
-  if {$err} { 
-    debug "gdbtk_busy ERROR: $txt" 
-  }
+  set e [BusyEvent \#auto]
+  GDBEventHandler::dispatch $e
+  delete object $e
 
   # Force the screen to update
   update
@@ -135,21 +149,41 @@ proc gdbtk_busy {} {
 #          that could change target state.
 # ------------------------------------------------------------------
 proc gdbtk_update {} {
-  set err [catch {run_hooks gdb_update_hook} txt]
-  if {$err} { 
-    debug "gdbtk_update ERROR: $txt" 
-  }
+
+  set e [UpdateEvent \#auto]
+  GDBEventHandler::dispatch $e
+  delete object $e
   
   # Force the screen to update
   update
 }
 
 # ------------------------------------------------------------------
-#   PROCEDURE: gdbtk_idle - run all idle hooks
+#   PROCEDURE:  gdbtk_update_safe - run all update hooks in a safe way
 #
-#          Use this procedure to run all the gdb_idle_hook's,
-#          which should free the UI for more user input. This
-#          hook should only be run AFTER all communication with
+#          Use this procedure to force all widgets to update
+#          themselves. This hook is usually run after command
+#          that could change target state.
+#          Like gdbtk_update but safe to be used in "after idle"
+#          which is used in update hooks.
+# ------------------------------------------------------------------
+proc gdbtk_update_safe {} {
+  global gdb_running
+
+  # Fencepost: Do not update if we are running the target
+  # We get here because script commands may have changed memory or
+  # registers and "after idle" events registered as a consequence
+  # If we try to update while the target is running we are doomed.
+  if {!$gdb_running} {
+    gdbtk_update
+  }
+}
+
+# ------------------------------------------------------------------
+#   PROCEDURE: gdbtk_idle - dispatch IdleEvent
+#
+#          Use this procedure to free the UI for more user input.
+#          This should only be run AFTER all communication with
 #          the target has halted, otherwise the risk of two (or
 #          more) widgets talking to the target arises.
 # ------------------------------------------------------------------
@@ -159,33 +193,35 @@ proc gdbtk_idle {} {
   # Put the unfiltered hook back in place, just in case
   # somebody swapped it out, and then died before they
   # could replace it.
-
   gdb_restore_fputs
-  set err [catch {run_hooks gdb_idle_hook} txt]
-  if {$err} { 
-    debug "gdbtk_idle 1 ERROR: $txt" 
+
+  set err [catch {run_hooks gdb_idle_hook}]
+  if {$err} {
+    dbug E "Error running gdb_idle_hook: $::errorInfo"
   }
-  
+
+  set e [IdleEvent \#auto]
+  GDBEventHandler::dispatch $e
+  delete object $e
+
   if {!$gdb_running} {
     set err [catch {run_hooks gdb_no_inferior_hook} txt]
     if {$err} { 
-      debug "gdbtk_idle 2 ERROR: $txt" 
+      dbug E "no_inferior_hook error: $txt" 
     }
-  }    
+  }
+
   # Force the screen to update
   update
 }
 
 define_hook download_progress_hook
 
-# Random hook of procs to call just before exiting.
-define_hook gdb_quit_hook
-
 # ------------------------------------------------------------------
 #  PROCEDURE:  gdbtk_quit_check - Ask if the user really wants to quit.
 # ------------------------------------------------------------------
 proc gdbtk_quit_check {} {
-  global gdb_downloading gdb_running
+  global gdb_downloading gdb_running gdb_exe_name
   
   if {$gdb_downloading} {
     set msg "Downloading to target,\n really close the debugger?"
@@ -193,28 +229,27 @@ proc gdbtk_quit_check {} {
       return 0
     }
   } elseif {$gdb_running} {
-    # While we are running the inferior, gdb_cmd is fenceposted and returns
-    # immediately. Therefore, we need to ask here. Do we need to stop the target,
-    # too?
+    # While we are running the inferior, gdb_cmd is fenceposted and
+    # returns immediately. Therefore, we need to ask here. Do we need
+    # to stop the target, too?
     set msg "A debugging session is active.\n"
     append msg "Do you still want to close the debugger?"
     if {![gdbtk_tcl_query $msg no]} {
       return 0
     }
   }
+  
   return 1
 }
 
 # ------------------------------------------------------------------
 #  PROCEDURE:  gdbtk_quit - Quit the debugger
 #         Call this procedure anywhere the user can request to quit.
-#         This procedure will ask all the right questions and run
-#         all the gdb_quit_hooks before exiting. 
+#         This procedure will ask all the right questions before
+#         exiting.
 # ------------------------------------------------------------------
 proc gdbtk_quit {} {
   if {[gdbtk_quit_check]} {
-    ManagedWin::shutdown
-    pref_save
     gdb_force_quit
   }
 }
@@ -224,8 +259,21 @@ proc gdbtk_quit {} {
 #         before exiting.  Last chance to cleanup!
 # ------------------------------------------------------------------
 proc gdbtk_cleanup {} {
-   # This is a sign that it is too late to be doing updates, etc...
-   set ::gdb_shutting_down 1
+  global gdb_exe_name
+
+  # Save the session
+  if {$gdb_exe_name != ""} {
+    session_save
+  }
+
+  # This is a sign that it is too late to be doing updates, etc...
+  set ::gdb_shutting_down 1
+
+  # Shutdown the window manager and save all preferences
+  # This way a "quit" in the console window will cause
+  # preferences to be saved.
+  ManagedWin::shutdown
+  pref_save
 }
 
 # ------------------------------------------------------------------
@@ -311,9 +359,20 @@ proc show_warning {message} {
   # here.
   set title "GDB"
   set modal "task"
+
+# On Windows, we use ide_messageBox which runs the Win32 MessageBox function
+# in another thread.  This permits a program which handles IDE requests from
+# other programs to not return from the request until the MessageBox completes.
+# This is not possible without using another thread, since the MessageBox
+# function call will be running its own event loop, and will be higher on the
+# stack than the IDE request.
+#
+# On Unix tk_messageBox runs in the regular Tk event loop, so
+# another thread is not required.
+
  
   if {$tcl_platform(platform) == "windows"} {
-      ide_messageBox -icon warning \
+      ide_messageBox [list set r] -icon warning \
         -default ok -message $message -title $title \
         -type ok -modal $modal -parent .
   } else {
@@ -352,15 +411,31 @@ proc echo {args} {
 }
 
 # ------------------------------------------------------------------
-# PROC: gdbtk_tcl_fputs_error -
+# PROC: gdbtk_tcl_fputs_error - write an error message
 # ------------------------------------------------------------------
 proc gdbtk_tcl_fputs_error {message} {
-  global gdbtk_state
-  # Restore the fputs hook, in case anyone forgot to put it back...
-  gdb_restore_fputs
+  if {$::gdbtk_state(console) != ""} {
+    $::gdbtk_state(console) einsert $message err_tag
+    update
+  }
+}
 
-  if {$gdbtk_state(console) != ""} {
-    $gdbtk_state(console) einsert $message
+# ------------------------------------------------------------------
+# PROC: gdbtk_tcl_fputs_log - write a log message
+# ------------------------------------------------------------------
+proc gdbtk_tcl_fputs_log {message} {
+  if {$::gdbtk_state(console) != ""} {
+    $::gdbtk_state(console) einsert $message log_tag
+    update
+  }
+}
+
+# ------------------------------------------------------------------
+# PROC: gdbtk_tcl_fputs_target - write target output
+# ------------------------------------------------------------------
+proc gdbtk_tcl_fputs_target {message} {
+  if {$::gdbtk_state(console) != ""} {
+    $::gdbtk_state(console) einsert $message target_tag
     update
   }
 }
@@ -388,26 +463,32 @@ proc gdbtk_tcl_end_variable_annotation {} {
 }
 
 # ------------------------------------------------------------------
-# PROC: gdbtk_tcl_breakpoint -
+# PROC: gdbtk_tcl_breakpoint - A breakpoint was changed -- notify
+#                               gui.
 # ------------------------------------------------------------------
-proc gdbtk_tcl_breakpoint {action bpnum addr line file bp_type enabled thread} {
-#  debug "BREAKPOINT: $action $bpnum $addr $line $file $bp_type $enabled $thread "
-  run_hooks gdb_breakpoint_change_hook $action $bpnum $addr $line $file $bp_type $enabled $thread
+proc gdbtk_tcl_breakpoint {action bpnum} {
+#  debug "BREAKPOINT: $action $bpnum"
+  set e [BreakpointEvent \#auto -action $action -number $bpnum]
+  GDBEventHandler::dispatch $e
+  delete object $e
 }
 
 # ------------------------------------------------------------------
-# PROC: gdbtk_tcl_tracepoint -
+# PROC: gdbtk_tcl_tracepoint - A tracepoint was changed -- notify
+#                               gui.
 # ------------------------------------------------------------------
-proc gdbtk_tcl_tracepoint {action tpnum addr line file pass_count} {
-#  debug "TRACEPOINT: $action $tpnum $addr $line $file $pass_count"
-  run_hooks gdb_breakpoint_change_hook $action $tpnum $addr $line $file tracepoint
+proc gdbtk_tcl_tracepoint {action tpnum} {
+#  debug "TRACEPOINT: $action $tpnum"
+  set e [TracepointEvent \#auto -action $action -number $tpnum]
+  GDBEventHandler::dispatch $e
+  delete object $e
 }
 
 # ------------------------------------------------------------------
 # PROC: gdbtk_tcl_trace_find_hook -
 # ------------------------------------------------------------------
 proc gdbtk_tcl_trace_find_hook {arg from_tty} {
-#  debug "Running trace find hook with $arg $from_tty"
+#  debug "$arg $from_tty"
   run_hooks gdb_trace_find_hook $arg $from_tty
 }
 
@@ -426,7 +507,7 @@ proc gdbtk_tcl_trace_find_hook {arg from_tty} {
 # ------------------------------------------------------------------
 proc gdb_run_readline_command {command args} {
   global gdbtk_state
-#  debug "run readline_command $command $args"
+#  debug "$command $args"
   set gdbtk_state(readlineArgs) $args
   gdb_cmd $command
 }
@@ -448,7 +529,7 @@ proc gdbtk_tcl_readline_begin {message} {
 # ------------------------------------------------------------------
 proc gdbtk_tcl_readline {prompt} {
   global gdbtk_state
-#  debug "gdbtk_tcl_readline $prompt"
+#  debug "$prompt"
   if {[info exists gdbtk_state(readlineArgs)]} {
     # Not interactive, so pop the list, and print element.
     set cmd [lvarpop gdbtk_state(readlineArgs)]
@@ -548,7 +629,7 @@ proc gdbtk_tcl_display {action number {value {}}} {
 #         the user has changed the contents of a register.
 # ------------------------------------------------------------------
 proc gdbtk_register_changed {} {
-  after idle gdbtk_update
+  after idle gdbtk_update_safe
 }
 
 # ------------------------------------------------------------------
@@ -558,7 +639,7 @@ proc gdbtk_register_changed {} {
 #         the program's variables).
 # ------------------------------------------------------------------
 proc gdbtk_memory_changed {} {
-  after idle gdbtk_update
+  after idle gdbtk_update_safe
 }
 
 ####################################################################
@@ -629,8 +710,17 @@ proc gdbtk_tcl_post_add_symbol {} {
 # ------------------------------------------------------------------
 proc gdbtk_tcl_file_changed {filename} {
 
-  SrcWin::point_to_main
-  run_hooks file_changed_hook
+  if {$filename == ""} {
+    gdb_clear_file
+    run_hooks gdb_clear_file_hook
+    set ::gdb_exe_name ""
+    set ::gdb_loaded 0
+    set ::gdb_running 0
+    gdbtk_update
+  } else {
+    SrcWin::point_to_main
+    run_hooks file_changed_hook
+  }
 }
 
 # ------------------------------------------------------------------
@@ -644,7 +734,7 @@ proc gdbtk_tcl_file_changed {filename} {
 #         commands, then we cannot look for main.
 # ------------------------------------------------------------------
 proc gdbtk_tcl_exec_file_display {filename} {
-  global gdb_loaded gdb_running gdb_exe_name gdb_target_changed
+  global gdb_exe_changed
 
   # DO NOT CALL set_exe here! 
 
@@ -657,7 +747,15 @@ proc gdbtk_tcl_exec_file_display {filename} {
 
   # set_exe calls file command with the filename in
   # quotes, so we need to strip them here.
-  set gdb_exe_name [string trim $filename \']
+  # We need to make sure that we turn filename into
+  # an absolute path or sessions won't work.
+  set filename [string trim $filename \']
+  if {[string index $filename 0] != "/"} {
+    set pwd [pwd]
+    set filename "$pwd/$filename"
+  }
+  set_exe_name $filename
+  set gdb_exe_changed 0
 
   SrcWin::point_to_main
 }
@@ -676,20 +774,33 @@ proc gdbtk_tcl_exec_file_display {filename} {
 #  3: source line number
 #  4: address
 #  5: current PC - which will often be the same as address, but not when
-#  6: shared library name if the pc is in a shared lib
 #  we are browsing, or walking the stack.
+#  6: shared library name if the pc is in a shared lib
 #
 # ------------------------------------------------------------------
 proc gdbtk_locate_main {} {
+  set result {}
   set main_names [pref get gdb/main_names]
-  debug "gdbtk_locate_main: Searching $main_names"
+  debug "Searching $main_names"
+
   foreach main $main_names {
     if {![catch {gdb_search functions $main -static 1}] \
         && ![catch {gdb_loc $main} linespec]} {
-      return $linespec
+      set result $linespec
+      break
     }
   }
-  return {}
+  if {$result == {} 
+      && ![catch gdb_entry_point entry_point]
+      && ![catch {gdb_loc "*$entry_point"} linespec]} {
+    set result $linespec
+  }
+  
+  # need to see if result is valid
+  lassign $result file func ffile line addr rest
+  if {$addr == 0x0 && $func == {}} { set result {} }
+
+  return $result
 }
 
 ##############################################
@@ -703,7 +814,12 @@ proc gdbtk_locate_main {} {
 # ------------------------------------------------------------------
 proc set_exe_name {exe} {
   global gdb_exe_name gdb_exe_changed
-  #debug "set_exe_name: exe=$exe  gdb_exe_name=$gdb_exe_name"
+  #debug "exe=$exe  gdb_exe_name=$gdb_exe_name"
+
+  if {$gdb_exe_name != ""} then {
+    session_save
+  }
+
   set gdb_exe_name $exe
   set gdb_exe_changed 1    
 }
@@ -720,7 +836,7 @@ proc set_exe {} {
     if {$gdb_exe_name == ""} { return }
     set err [catch {gdb_cmd "file '$gdb_exe_name'" 1} msg]
     if {$err} {
-      debug "set_exe ERROR: $msg"
+      dbug E "$msg"
       set l [split $msg :]
       set errtxt [join [lrange $l 1 end] :]
       set msg "Error loading \"$gdb_exe_name\":\n"
@@ -822,6 +938,36 @@ proc _open_file {{file ""}} {
   return 1
 }
 
+# ------------------------------------------------------------------
+#  _close_file - close the current executable and prepare for
+#    another executable.
+# ------------------------------------------------------------------
+proc _close_file {} {
+
+  # If there is already an inferior, ask him if he wants to close
+  # the file. If there is already an exec file loaded (and not run)
+  # also ask, but don't ask twice.
+  set okay 1
+  if {[gdb_target_has_execution]} {
+    set okay [gdbtk_tcl_query "Program is already running.\nClose file anyway?"]
+  } elseif {$::gdb_exe_name != ""} {
+    set okay [gdbtk_tcl_query "Program already loaded.\nClose file anyway?"]
+  } else {
+    # No exec file yet
+    return
+  }
+
+  if {$okay} {
+    gdb_clear_file
+    gdbtk_tcl_file_changed ""
+
+    # Print out a little message to all console windows
+    foreach cw [ManagedWin::find Console] {
+      $cw insert "No executable file now.\n"
+    }
+  }
+}
+
 # ------------------------------------------------------------------
 # PROC: set_target_name - Update the target name.  
 #
@@ -945,7 +1091,7 @@ necessary,\nmodify the port setting with the debugger preferences."
     
     if {![catch {pref get gdb/load/$gdb_target_name-after_attaching} aa] && $aa != ""} {
       if {[catch {gdb_cmd $aa} err]} {
-	catch {[ManagedWin::find Console] einsert $err}
+	catch {[ManagedWin::find Console] einsert $err err_tag}
       }
     }
     set gdb_target_changed 0
@@ -963,7 +1109,7 @@ necessary,\nmodify the port setting with the debugger preferences."
 proc run_executable { {auto_start 1} } {
   global gdb_loaded gdb_downloading gdb_target_name
   global gdb_exe_changed gdb_target_changed gdb_program_has_run
-  global gdb_running gdb_exe_name
+  global gdb_running gdb_exe_name tcl_platform
 
 #  debug "auto_start=$auto_start gdb_target_name=$gdb_target_name"
 
@@ -1052,10 +1198,17 @@ proc run_executable { {auto_start 1} } {
       set gdb_args [pref getd gdb/load/$gdb_target_name-opts]
       if { $gdb_args != ""} {
 	debug "set args $gdb_args"
-	catch {gdb_cmd "set args $gdb_args"}
+	gdb_set_inferior_args $gdb_args
       }
     }
 
+    # If the user requested it, start an xterm for use as the
+    # inferior's tty.
+    if {$tcl_platform(platform) != "windows"
+	&& [pref getd gdb/process/xtermtty] == "yes"} {
+      tty::create
+    }
+
     # 
     # Run
 
@@ -1159,6 +1312,66 @@ proc gdbtk_attach_remote {} {
   return $r
 }
 
+# ------------------------------------------------------------------
+# PROC:  gdbtk_connect: connect to a remote target 
+#                      in asynch mode if async is 1
+# ------------------------------------------------------------------
+proc gdbtk_connect {{async 0}} {
+  global file_done
+
+  debug "async=$async"
+
+  gdbtk_busy
+
+  set result [gdbtk_attach_remote]
+  switch $result {
+    ATTACH_ERROR {
+      set successful 0
+    }
+
+    ATTACH_TARGET_CHANGED {
+	if {[pref get gdb/load/check] && $file_done} {
+	  set err [catch {gdb_cmd "compare-sections"} errTxt]
+	  if {$err} {
+	    set successful 0
+	    tk_messageBox -title "Error" -message $errTxt \
+	      -icon error -type ok
+	    break
+	  }
+	}
+
+	tk_messageBox -title "GDB" -message "Successfully connected" \
+	  -icon info -type ok
+	set successful 1
+    }
+
+    ATTACH_CANCELED {
+	tk_messageBox -title "GDB" -message "Connection Canceled" -icon info \
+	  -type ok
+	set successful 0
+    }
+
+    ATTACH_TARGET_UNCHANGED {
+	tk_messageBox -title "GDB" -message "Successfully connected" \
+	  -icon info -type ok
+	set successful 1
+    }
+
+    default {
+	dbug E "Unhandled response from gdbtk_attach_remote: \"$result\""
+	set successful 0
+    }
+  }
+
+  gdbtk_idle
+
+  # Whenever we attach, we need to do an update
+  if {$successful} {
+    gdbtk_attached
+  }
+  return $successful
+}
+
 # ------------------------------------------------------------------
 #  PROC: gdbtk_step - step the target
 # ------------------------------------------------------------------
@@ -1358,9 +1571,9 @@ proc do_state_hook {varname ind op} {
 }
 
 # ------------------------------------------------------------------
-# PROC: disconnect -
+# PROC: gdbtk_disconnect -
 # ------------------------------------------------------------------
-proc disconnect {{async 0}} {
+proc gdbtk_disconnect {{async 0}} {
    global gdb_loaded gdb_target_changed
    catch {gdb_cmd "detach"}
    # force a new target command to do something
@@ -1454,7 +1667,7 @@ define_hook gdb_clear_file_hook
 proc gdbtk_clear_file {} {
   global gdb_target_name
 
-  dbug W "GDBTK_CLEAR_FILE"
+  debug
   # Give widgets a chance to clean up
   run_hooks gdb_clear_file_hook
 
